@@ -28,12 +28,35 @@ from .validators import assert_bus_name_valid, assert_object_path_valid
 MESSAGE_TYPE_CALL = MessageType.METHOD_CALL
 MESSAGE_TYPE_SIGNAL = MessageType.SIGNAL
 NO_REPLY_EXPECTED_VALUE = MessageFlag.NO_REPLY_EXPECTED.value
+_LOGGER = logging.getLogger(__name__)
 
 
-def _expects_reply(msg) -> bool:
+_Message = Message
+
+
+def _expects_reply(msg: _Message) -> bool:
+    """Whether a message expects a reply."""
     return not (msg.flags.value & NO_REPLY_EXPECTED_VALUE)
 
 
+def _block_unexpected_reply(reply: _Message) -> None:
+    """Block a reply if it's not expected.
+
+    Previously we silently ignored replies that were not expected, but this
+    lead to implementation errors that were hard to debug. Now we log a
+    debug message instead.
+    """
+    _LOGGER.debug(
+        "Blocked attempt to send a reply from handler "
+        "that received a message with flag "
+        "MessageFlag.NO_REPLY_EXPECTED: %s",
+        reply,
+    )
+
+
+BLOCK_UNEXPECTED_REPLY = _block_unexpected_reply
+
+
 class SendReply:
     """A context manager to send a reply to a message."""
 
@@ -48,8 +71,7 @@ class SendReply:
         return self
 
     def __call__(self, reply: Message) -> None:
-        if _expects_reply(self._msg):
-            self._bus.send(reply)
+        self._bus.send(reply)
 
     def _exit(
         self,
@@ -57,23 +79,21 @@ class SendReply:
         exc_value: Optional[Exception],
         tb: Optional[TracebackType],
     ) -> bool:
-        if exc_type is None:
-            return False
-
-        if issubclass(exc_type, DBusError):
-            self(exc_value._as_message(self._msg))  # type: ignore[union-attr]
-            return True
-
-        if issubclass(exc_type, Exception):
-            self(
-                Message.new_error(
-                    self._msg,
-                    ErrorType.SERVICE_ERROR,
-                    f"The service interface raised an error: {exc_value}.\n{traceback.format_tb(tb)}",
+        if exc_value:
+            if isinstance(exc_value, DBusError):
+                self(exc_value._as_message(self._msg))
+            else:
+                self(
+                    Message.new_error(
+                        self._msg,
+                        ErrorType.SERVICE_ERROR,
+                        f"The service interface raised an error: {exc_value}.\n{traceback.format_tb(tb)}",
+                    )
                 )
-            )
             return True
 
+        return False
+
     def __exit__(
         self,
         exc_type: Optional[Type[Exception]],
@@ -804,7 +824,7 @@ class BaseMessageBus:
                 ErrorType.INTERNAL_ERROR, "invalid message type for method call", msg
             )
 
-    def _process_message(self, msg) -> None:
+    def _process_message(self, msg: _Message) -> None:
         handled = False
         for user_handler in self._user_message_handlers:
             try:
@@ -855,9 +875,19 @@ class BaseMessageBus:
         if msg.message_type is MESSAGE_TYPE_CALL:
             if not handled:
                 handler = self._find_message_handler(msg)
+                if not _expects_reply(msg):
+                    if handler:
+                        handler(msg, BLOCK_UNEXPECTED_REPLY)
+                    else:
+                        _LOGGER.error(
+                            '"%s.%s" with signature "%s" could not be found',
+                            msg.interface,
+                            msg.member,
+                            msg.signature,
+                        )
+                    return
 
                 send_reply = SendReply(self, msg)
-
                 with send_reply:
                     if handler:
                         handler(msg, send_reply)
@@ -881,69 +911,82 @@ class BaseMessageBus:
     def _make_method_handler(
         self, interface: ServiceInterface, method: _Method
     ) -> Callable[[Message, Callable[[Message], None]], None]:
-        def handler(msg: Message, send_reply: Callable[[Message], None]) -> None:
-            args = ServiceInterface._msg_body_to_args(msg)
-            result = method.fn(interface, *args)
-            body, fds = ServiceInterface._fn_result_to_body(
+        method_fn = method.fn
+        out_signature_tree = method.out_signature_tree
+        negotiate_unix_fd = self._negotiate_unix_fd
+        out_signature = method.out_signature
+        message_type_method_return = MessageType.METHOD_RETURN
+        msg_body_to_args = ServiceInterface._msg_body_to_args
+        fn_result_to_body = ServiceInterface._fn_result_to_body
+
+        def _callback_method_handler(
+            msg: Message, send_reply: Callable[[Message], None]
+        ) -> None:
+            result = method_fn(interface, *msg_body_to_args(msg))
+            if send_reply is BLOCK_UNEXPECTED_REPLY or not _expects_reply(msg):
+                return
+            body, fds = fn_result_to_body(
                 result,
-                signature_tree=method.out_signature_tree,
-                replace_fds=self._negotiate_unix_fd,
+                signature_tree=out_signature_tree,
+                replace_fds=negotiate_unix_fd,
             )
             send_reply(
                 Message(
-                    message_type=MessageType.METHOD_RETURN,
+                    message_type=message_type_method_return,
                     reply_serial=msg.serial,
                     destination=msg.sender,
-                    signature=method.out_signature,
+                    signature=out_signature,
                     body=body,
                     unix_fds=fds,
                 )
             )
 
-        return handler
+        return _callback_method_handler
 
     def _find_message_handler(
-        self, msg
-    ) -> Optional[Callable[[Message, Callable], None]]:
-        handler: Optional[Callable[[Message, Callable], None]] = None
-
+        self, msg: _Message
+    ) -> Optional[Callable[[Message, Callable[[Message], None]], None]]:
         if (
             msg.interface == "org.freedesktop.DBus.Introspectable"
             and msg.member == "Introspect"
             and msg.signature == ""
         ):
-            handler = self._default_introspect_handler
+            return self._default_introspect_handler
 
-        elif msg.interface == "org.freedesktop.DBus.Properties":
-            handler = self._default_properties_handler
+        if msg.interface == "org.freedesktop.DBus.Properties":
+            return self._default_properties_handler
 
-        elif msg.interface == "org.freedesktop.DBus.Peer":
+        if msg.interface == "org.freedesktop.DBus.Peer":
             if msg.member == "Ping" and msg.signature == "":
-                handler = self._default_ping_handler
+                return self._default_ping_handler
             elif msg.member == "GetMachineId" and msg.signature == "":
-                handler = self._default_get_machine_id_handler
-        elif (
+                return self._default_get_machine_id_handler
+
+        if (
             msg.interface == "org.freedesktop.DBus.ObjectManager"
             and msg.member == "GetManagedObjects"
         ):
-            handler = self._default_get_managed_objects_handler
+            return self._default_get_managed_objects_handler
 
-        elif msg.path:
-            for interface in self._path_exports.get(msg.path, []):
-                for method in ServiceInterface._get_methods(interface):
+        msg_path = msg.path
+        if msg_path:
+            interfaces = self._path_exports.get(msg_path)
+            if not interfaces:
+                return None
+            for interface in interfaces:
+                methods = ServiceInterface._get_methods(interface)
+                for method in methods:
                     if method.disabled:
                         continue
+
                     if (
                         msg.interface == interface.name
                         and msg.member == method.name
                         and msg.signature == method.in_signature
                     ):
-                        handler = ServiceInterface._get_handler(interface, method, self)
-                        break
-                if handler:
-                    break
+                        return ServiceInterface._get_handler(interface, method, self)
 
-        return handler
+        return None
 
     def _default_introspect_handler(
         self, msg: Message, send_reply: Callable[[Message], None]
